/*
 * 12_Drive_Analogue.cpp
 * Created: 2020
 *
 * Pindescription Arduino Uno
 * No.	Func.	PIN-MC	Used as In/Out Res. Pindefinition
 * 0	RX		PD0		RX		In
 * 1	TX		PD1		TX		In
 * 2	INT0	PD2		US-Echo	In
 * 3	Digit	PD3		Servo	Out
 * 4	Digit	PD4		Line-M	In
 * 5	OC0B	PD5		PWM-A	Out
 * 6	OC0B	PD6		PWM-B	Out
 * 7	PCINT23	PD7		Color-1	In	->	DDRD = 0b01101000
 * 8	Digit	PB0		Motor-A	Out
 * 9	Digit	PB1		Motor-B	Out
 *10	Digit	PB2		Line-L	In
 *11	PCINT3	PB3		Color-2	In
 *12	Digit	PB4		Infr.	In	Not needed
 *13	LED		PB5		LED		Out	->	DDRB = 0b00100011
 *A0	Ana		PC0		Line-RA	In
 *A1	Ana		PC1		Line-LA	In
 *A2	Ana		PC2		Dist-1	In
 *A3	Ana		PC3		Dist-2	In
 *A4	Digit	PC4		Line-R	In
 *A5	Digit	PC5		US-Trig	Out	->	DDRC = 0b00100000
 */ 
#define F_CPU 16000000UL //16MHz required for delay
#include <avr/io.h> //Input/Output library
#include <util/delay.h> //Needed for using _delay_...
#include <avr/interrupt.h> //External and internal Interrupts
#include "Init.h"
#include "USART_Functions.h"
#include "Drive_Functions.h"
#include "Interrupt_Service_Routines.h"

//Definitions
#define FOSC				16000000 // Clock Speed
#define BAUD				38400//Baud rate, normally used by Arduino 9600, possible rates: n * 9600
#define MYUBRR				FOSC/16/BAUD-1 //Calculate my baud rate
#define	PWMA				OCR0A //Overflow compare register A
#define PWMB				OCR0B //Overflow compare register B
//#define hundred				0 //First digit of a 3 digit literal
//#define ten					1 //Second digit
//#define one					2 //Third digit
#define Line_analogue_right	Analogue_value[0] //Right line sensor
#define Line_analogue_left	Analogue_value[1] //Left line sensor
#define Distance_1			Analogue_value[2] //UV distance sensor
#define Distance_2			Analogue_value[3] //UV distance sensor

//Macros
#define	LED_ON				PORTB |= (1 << PORTB5) //Push PORTB pin 5 high (LED, pin 13 "Uno")
#define	LED_OFF				PORTB &= ~(1 << PORTB5) //~ Not Operator
//#define	LED_TOGGLE			PINB |= (1<<PINB5) //Toggle Arduino Uno pin 13


//Variables
unsigned char z; //For tests

//unsigned int T2ck; //Counter TMR2 overflow
//unsigned char Line_digital_right; //State of right line sensor
//unsigned char Line_digital_middle; //State of middle line sensor
//unsigned char Line_digital_left; //State of left line sensor
unsigned char Correction_Value;

//unsigned char US_Time; //Runtime of US signal, HI part
//unsigned char Counter_US; //Counter TMR2 overflow generating a US-trigger every 32ms

//unsigned char Counter_Transmission; //Counter of digits of a transmission of a literal
//unsigned char Counter_Transmission_Data; //Counter of transmitted data

//unsigned char Data[10]; //Number of datas displayed on Data Visualizer
//unsigned char Trans_data[3]; //3 digits who will be transmitted

//unsigned char Channel; //Analogue channel of the ADC
//unsigned char Analogue_value[4]; //Number of analogue inputs

//Funktions, declaration
void Init (void); //Initialize Microcontroller
//void USART_Transmit(unsigned char); //Transmit ASCII to Data Visualizer
void USART_Init(unsigned int); //Initialize USART Transmission/Reception
void Transmit_literal (char); //Transmit literal to Data Visualizer Monitor
void Computing_Transmission_Values (unsigned char); //Compute the 3 digits of a literal
void Data_Visualizer (void); //Define datas to be vizualized

//void Forward (unsigned char, unsigned char); //Direction with velocity A and B,for driving straight and driving a curve
//void Backward (unsigned char, unsigned char); //Direction with velocity A and B,for driving straight and driving a curve
//void Turn_right (unsigned char); //Rotate on the spot right, same velocity of the wheels
//void Turn_left (unsigned char); //Rotate on the spot left, same velocity of the wheels
//void Stop (void); //Stop all Motors


//Main routine
int main(void) {
	Init(); //Initalize the MC
	USART_Init(MYUBRR); //Transmision/reception initialize
	sei(); //Enable all interrupts
	while (0) {
		Turn_right(128);
	}
/************************************************************************/
/* Line   (Comment out the lines of "Maze)                                */
/************************************************************************/
	//while (1) {
		//Data_Visualizer(); //Define the values to be displayed 
		//if (US_Time > 10) { //No obstacle
			//if (Line_analogue_left < Line_analogue_right) { //Possibly add correction value
				////Sure, there is a better equation! You have to experiment!!
				//Correction_Value = (Line_analogue_right - Line_analogue_left) >> 1; //You have to set the parameters!
				//Forward(128 - Correction_Value, 128); //Take a left turn
			//}
			//else {
				//Correction_Value = (Line_analogue_left - Line_analogue_right) >> 1; //You have to set the parameters!
				//Forward(128, 128 - Correction_Value); //Take a right turn
			//}
		//}
		//else { //There is an obstacle
			//Stop(); //Of course you may change the behavior
		//}
	//}	
/************************************************************************/
/* Maze  (Comment out the lines of "Line")                                 */
/************************************************************************/
Stop();
while (1) {
	Data_Visualizer();
}
	while (0) {
		Data_Visualizer(); //Define the values to be displayed 
		if (US_Time > 10) { //No wall in the Front?
			if (Distance_1 > 80) { //Too close to the wall. The 80 are an example!! You have to set the parameters!!
				Forward(100, 150); //Take a left turn, you have to set the parameters!
			}
			else {
				if (Distance_1 < 60) { //Too far away from the wall, see above.
					Forward(150, 100); //Take a right turn, you have to set the parameters!
				}
				else {
					Forward(128, 128); //You are right, drive straight on
				}
			}
		}
		else { //There is a wall in the front
			Stop(); //Of course you may change the behavior
		}
	}
}

/***********************************************************************************************/
//Other functions

//Data beeing transmitted to the Data Visualizer
void Data_Visualizer (void) {
	Data[0] = Line_digital_right;
	Data[1] = Line_digital_middle;
	Data[2] = Line_digital_left;
	Data[3] = Line_analogue_right;
	Data[4] = Line_analogue_left;
	//	Data[5] = Color_Value_1;
	//	Data[6] = Color_Value_2;
	Data[7] = Distance_1;
	Data[8] = Distance_2;
	Data[9] = US_Time;
}

/***************************************************************************************/




