/*
 * Interrupt_Service_Routines.h
 *
 * 2020
 */ 


#ifndef INTERRUPT_SERVICE_ROUTINES_H_
#define INTERRUPT_SERVICE_ROUTINES_H_

//Interrupt service routines (ISR)

#define	LED_TOGGLE			PINB |= (1<<PINB5) //Toggle Arduino Uno pin 13

unsigned char Line_digital_right; //State of right line sensor
unsigned char Line_digital_middle; //State of middle line sensor
unsigned char Line_digital_left; //State of left line sensor

unsigned int T2ck; //Counter TMR2 overflow

unsigned char US_Time; //Runtime of US signal, HI part
unsigned char Counter_US; //Counter TMR2 overflow generating a US-trigger every 32ms

unsigned char Counter_Transmission; //Counter of digits of a transmission of a literal
unsigned char Counter_Transmission_Data; //Counter of transmitted data

unsigned char Data[10]; //Number of datas displayed on Data Visualizer

unsigned char Channel; //Analogue channel of the ADC
unsigned char Analogue_value[4]; //Number of analogue inputs
unsigned char AD_Counter;

//TMR2 overflow Interrupt
//To time several works 
ISR(TIMER2_OVF_vect) { //Prescaler TMR2 1:8 = Interrupt every 128 us
	AD_Counter++;
	if (AD_Counter == 4) {
		AD_Counter = 0;
		ADMUX = 0b01100000 | (Channel); //Bit 3:0 choose the channel
		ADCSRA |= (1 << ADSC); //Start conversion
	}
//Toggeling the LED, just to see, if the MC is working
	T2ck++; //Counter TMR2 OVF
	if(T2ck == 2000) { //Every 250ms
		T2ck = 0; //Dont forget!!
		LED_TOGGLE; //See above
	}
//Ultrasonic distance
	Counter_US++; //256 * 128us = 32768us
	if (!Counter_US) {//Approx. every 32ms
//		LED_TOGGLE; //For testing
		PORTC |= 0b00100000; //Trigger US, HI
		_delay_us(10); //10 microseconds
		PORTC &= 0b11011111; //Trigger US, LO
		//First you have to write TCNT1H, before writting TCNT1L!!!
		TCNT1H = 0; //Timer 1 set to zero...
		TCNT1L = 0; //...to quantify the distance of the obstacle
		//LED_ON; //Tests
	}
//Transmitting data	
//Its a little bit tricky, but it saves a plenty of time
	if (T2ck & 0b00000001) { //Every 256us,
		//**********************needs 2us per digit**********************************
		//instead of over 100us, while waiting for transmitting byte
//		LED_ON; //Test
		Counter_Transmission++; //Four digits will be transmitted
		if (Counter_Transmission == 4) { //
			Counter_Transmission = 0; //Modulo 4
			Counter_Transmission_Data++;  //
			if (Counter_Transmission_Data == 10) { //There will be transmitted ten literals
				Counter_Transmission_Data = 0; //Modulo 10
				USART_Transmit('\n'); //At last write a new line
			}
			Computing_Transmission_Values(Data[Counter_Transmission_Data]); //Computing the next three digits
		}
		//Only initialize, there is no need waiting, while transmission is done
		if (Counter_Transmission == 3) USART_Transmit('\t'); //At last write a tabulator
		else USART_Transmit(Trans_data[Counter_Transmission]); //One digit after another
//		LED_OFF; //Test
	}
//States of Line Detector
	Line_digital_left = PINC & 0b00010000; //State of PORTC 4 pin, if the pin is HI, 16 will be displayed
	Line_digital_middle = PIND &0b00010000; //PortD 4 pin, display 16
	Line_digital_right = PINB & 0b00000100; //PORTB 2 pin, display 4
}

//Interrupt on falling edge at RD0, US sensor
ISR(INT0_vect) {
	//LED_ON; //Test
	unsigned char junk; //Only to read TCNT1L
	junk = TCNT1L; //We have to read the low Byte first, only then we could read TCN1H, but we dont need the value
	US_Time = TCNT1H; //runtime of the US signal, HI part
	TCNT1H = 0; //Reset TMR1
	TCNT1L = 0; //Reset
	//16MHz Prescaler 1:8 -> 2MHz TMR1 Counter
	//LED_OFF; //Test
}

//Timer 1 Overflow Interrupt
ISR(TIMER1_OVF_vect) { //Prescaler TMR1 1:8 = Interrupt every 32 ms
	US_Time = 255; //If there is no obstacle
	//LED_TOGGLE; //for testing
}

//Interrupt on completition of Analog_to_Digital_Conversion
ISR(ADC_vect) {
	Analogue_value[Channel] = ADCH; //HI byte of the resolution
	Channel++; //Channels of the ADC
	if (Channel == 4) Channel = 0; //We only need 4 channels of 6 of the Atmega 328P
}

#endif /* INTERRUPT_SERVICE_ROUTINES_H_ */